/* 
	Copyright Miloslav Semler & Monokrystaly, s.r.o
	This software is provided as-is without any warranty.
*/
#define _BSD_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <string.h>

int serial_open(const char *name){
	int fd;
	struct termios newio;
	
	fd = open(name, O_RDWR|O_NOCTTY|O_NDELAY|O_NONBLOCK);
	if(fd < 0){
		return -1;
	}
	
	tcgetattr(fd, &newio);

	//newio.c_cflag |= CS8 |  CLOCAL | CREAD;
	//newio.c_cflag |= CS8 |  CLOCAL | CREAD | CRTSCTS;
	newio.c_cflag |= CS8 |  CLOCAL | CREAD;
	newio.c_cflag &= ~PARENB;
	//newio.c_ispeed = B9600;
	//newio.c_ospeed = B9600;
	/* pri vyndani/zandani USB se neresetuje nastaveni, proto 2400, pak 9600 */
	cfsetspeed(&newio, B2400);
	cfsetspeed(&newio, B9600);
	
	newio.c_iflag |= IGNPAR | IGNCR;
	newio.c_iflag &= ~(IXON | IXOFF | IXANY);
	newio.c_oflag &= ~OPOST; // raw output
	newio.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	newio.c_cc[VMIN] = 0;
	newio.c_cc[VTIME] = 0;
	
	if(tcsetattr(fd, TCSANOW, &newio) == -1){
		return -1;
	}else{
		return fd;
	}
}

int setRTS(int fd, int level)
{
    int status;

    if (ioctl(fd, TIOCMGET, &status) == -1) {
        perror("setRTS(): TIOCMGET");
        return 0;
    }
    if (level)
        status |= TIOCM_RTS;
    else
        status &= ~TIOCM_RTS;
    if (ioctl(fd, TIOCMSET, &status) == -1) {
        perror("setRTS(): TIOCMSET");
        return 0;
    }
    return 1;
}

int do_command(char *reply_buff, size_t reply_buff_sz, const char *cmd, int fd){
	int cnt = 0, cycles = 0, nread, rc;
	struct timeval tm;
	fd_set input;
	
	//setRTS(fd, 0);
	if(write(fd, cmd, strlen(cmd)) != strlen(cmd)){
		reply_buff[0] = 0;
		return -1;
	}
	tcdrain(fd);
	//setRTS(fd, 1);
	
	while(cnt < (reply_buff_sz -1) && cycles < 10){
		tm.tv_sec = 0;
		tm.tv_usec = 100000;
		FD_ZERO(&input);
		FD_SET(fd, &input);
		rc = select(fd + 1, &input, NULL, NULL, &tm);
		if(rc > 0){
			nread = read(fd, reply_buff + cnt, reply_buff_sz - cnt);
			cnt += nread;
		}else if(rc == 0){
			cycles++;
		}else{
			reply_buff[cnt] = 0;
			return -1;
		}
		if(memchr(reply_buff, '\n', cnt) != NULL){
			reply_buff[cnt] = 0;
			return 0;
		}
	}
	reply_buff[cnt] = 0;
	return -1;
}
